var REINDEX_ACTION = function ( rec, full ) {
  var indexUrl = null;
  if ( full ) {
    indexUrl = Sonatype.config.servicePath + '/data_index';
  } else {
    indexUrl = Sonatype.config.servicePath + '/data_incremental_index';
  }

  var url = indexUrl +
    rec.data.resourceURI.slice(Sonatype.config.host.length + Sonatype.config.servicePath.length);
  
  //make sure to provide /content path for repository root requests like ../repositories/central
  if (/.*\/repositories\/[^\/]*$/i.test(url) || /.*\/repo_groups\/[^\/]*$/i.test(url)){
    url += '/content';
  }
  
  Ext.Ajax.request({
    url: url,
    callback: function(options, isSuccess, response) {
      if ( !isSuccess ) {
        Sonatype.utils.connectionError( response, 'The server did not re-index the repository.' );
      }
    },
    scope: this,
    method: 'DELETE'
  });
}
  
Sonatype.Events.addListener( 'repositoryMenuInit',
  function( menu, repoRecord ) {
    if ( Sonatype.lib.Permissions.checkPermission( 'nexus:index', Sonatype.lib.Permissions.DELETE ) 
        && repoRecord.get( 'repoType' ) != 'virtual' ) {
      menu.add({
        text: 'ReIndex',
        handler: function( rec ) {
          REINDEX_ACTION( rec, true );
        },
        scope: this
      });
      menu.add( {
        text: 'Incremental ReIndex',
        handler: function( rec ) {
          REINDEX_ACTION( rec, false );
        },
        scope: this
      });
    }
  }
);

Sonatype.Events.addListener( 'repositoryContentMenuInit',
  function( menu, repoRecord, contentRecord ) {
    if ( Sonatype.lib.Permissions.checkPermission( 'nexus:index', Sonatype.lib.Permissions.DELETE ) 
        && repoRecord.get( 'repoType' ) != 'virtual' ) {
      menu.add({
        text: 'ReIndex',
        handler: function( rec ) {
          REINDEX_ACTION( rec, true );
        },
        scope: this
      });
      menu.add( {
        text: 'Incremental ReIndex',
        handler: function( rec ) {
          REINDEX_ACTION( rec, false );
        },
        scope: this
      });
    }
  }
);
/*
 * Sonatype Nexus (TM) Open Source Version. Copyright (c) 2008 Sonatype, Inc.
 * All rights reserved. Includes the third-party code listed at
 * http://nexus.sonatype.org/dev/attributions.html This program is licensed to
 * you under Version 3 only of the GNU General Public License as published by
 * the Free Software Foundation. This program is distributed in the hope that it
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License Version 3 for more details. You should have received a copy of
 * the GNU General Public License Version 3 along with this program. If not, see
 * http://www.gnu.org/licenses/. Sonatype Nexus (TM) Professional Version is
 * available from Sonatype, Inc. "Sonatype" and "Sonatype Nexus" are trademarks
 * of Sonatype, Inc.
 */
Sonatype.repoServer.SearchPanel = function(config) {
  var config = config || {};
  var defaultConfig = {};
  Ext.apply(this, config, defaultConfig);

  this.grid = new Sonatype.repoServer.SearchResultGrid( {
    searchPanel : this
  });

  this.searchTypes = [];

  // fire event for plugins to add their own search items
  Sonatype.Events.fireEvent('searchTypeInit', this.searchTypes, this);

  // no items, no page
  if (this.searchTypes.length < 1)
  {
    return;
  }
  
  var defaultSearchTypeIndex = 0;
  // find default
  for (var i = 0; i < this.searchTypes.length; i++)
  {
    if (this.searchTypes[i].defaultQuickSearch)
    {
      defaultSearchTypeIndex = i;
      break;
    }
  }

  this.searchTypeButton = new Ext.Button( {
    text : this.searchTypes[defaultSearchTypeIndex].text,
    value : this.searchTypes[defaultSearchTypeIndex].value,
    tooltip : 'Click for more search options',
    handler : this.switchSearchType,
    scope : this,
    menu : {
      items : this.searchTypes
    }
  });

  this.searchToolbar = new Ext.Toolbar( {
    ctCls : 'search-all-tbar',
    items : [ this.searchTypeButton, this.convertToFieldObject(this.searchTypes[defaultSearchTypeIndex].panelItems[0]) ]
  });

  this.artifactContainer = new Sonatype.repoServer.ArtifactContainer( {});

  Sonatype.repoServer.SearchPanel.superclass.constructor.call(this, {
    layout : 'border',
    hideMode : 'offsets',
    tbar : this.searchToolbar,
    items : [ this.grid, this.artifactContainer ]
  });

  this.grid.getSelectionModel().on('rowselect', this.displayArtifactInformation, this);
  this.grid.clearButton.on('click', this.clearArtifactInformation, this);
};

Ext.extend(Sonatype.repoServer.SearchPanel, Ext.Panel, {
  clearArtifactInformation : function(button, e) {
    this.artifactContainer.collapsePanel();
  },

  displayArtifactInformation : function(selectionModel, index, rec) {
    var searchType = this.getSearchType(this.searchTypeButton.value);
        if (typeof searchType.showArtifactContainer != 'function' || searchType.showArtifactContainer(rec))
    {
      this.artifactContainer.updateArtifact(rec.data);
    }
  },
  // search type switched on the drop down button
  switchSearchType : function(button, event) {
    // if event is null, this is called directly, and we
    // we will reset regardless if already selected, otherwise
    // no need to do anything if already set to same value
        if (event == null || this.searchTypeButton.value != button.value)
        {
      this.searchTypeButton.value = button.value;
      this.searchTypeButton.setText(this.getSearchType(button.value).text);
      this.clearWarningLabel();
      this.loadSearchPanel();
      this.switchStore();
    }
  },
  // load the dynamic panel
  loadSearchPanel : function() {
    // first remove current items
        while (this.searchToolbar.items.length > 1)
        {
      var item = this.searchToolbar.items.last();
      this.searchToolbar.items.remove(item);
      item.destroy();
    }

    // now add the other items
    var searchType = this.getSearchType(this.searchTypeButton.value);

        if (searchType != null)
        {
          for (var i = 0; i < searchType.panelItems.length; i++)
          {
        // can't simply add object config to toolbar, need to create
        // a real item
        this.searchToolbar.add(this.convertToFieldObject(searchType.panelItems[i]));
      }
    }
  },
  // toolbar only supports adding certain types of items, so we
  // need to do some special handling
  convertToFieldObject : function(config) {
        if (config.xtype == 'nexussearchfield')
        {
      return new Ext.app.SearchField(config);
        }
        else if (config.xtype == 'textfield')
        {
      return new Ext.form.TextField(config);
        }
        else
        {
      return config;
    }
  },
  // different search types may have different stores
  switchStore : function() {
    var searchType = this.getSearchType(this.searchTypeButton.value);
    this.grid.switchStore(this.grid, searchType.store, searchType.columnModel);
  },
  // retrieve the specified search type object
  getSearchType : function(value) {
        for (var i = 0; i < this.searchTypes.length; i++)
        {
          if (this.searchTypes[i].value == value)
          {
        return this.searchTypes[i];
      }
    }

    return null;
  },
  // set the warning in the toolbar
  setWarningLabel : function(s) {
    this.clearWarningLabel();
    this.warningLabel = this.searchToolbar.addText('<span class="x-toolbar-warning">' + s + '</span>');
  },
  // clear the warning in the toolbar
  clearWarningLabel : function() {
        if (this.warningLabel)
        {
      this.warningLabel.destroy();
      this.warningLabel = null;
    }
  },
  // start the search
  startSearch : function(panel, updateHistory) {
        if (updateHistory)
        {
      // update history in address bar of browser
      panel.extraData = null;
      Sonatype.utils.updateHistory(panel);
    }

    var searchType = this.getSearchType(this.searchTypeButton.value);
    
        if (panel.grid.store.sortInfo)
        {
      panel.grid.store.sortInfo = null;
      panel.grid.getView().updateHeaders();
    }

    searchType.searchHandler.call(this, panel);
  },
  // get the records from the server using grid
  fetchRecords : function(panel, reverse) {
    panel.artifactContainer.collapsePanel();
    panel.grid.totalRecords = 0;
    panel.grid.store.removeAll();
    panel.grid.store.load();
    
        if (reverse)
        {
      panel.grid.store.on( 'load', this.sortResults, this );
    }
        else
        {
      panel.grid.store.un( 'load', this.sortResults, this );
    }
  },
  sortResults : function( store, records, options ) {
    store.sort( 'version', 'desc' );
  },
  // start the quick search, we will look at all search types
  // and try to guess which type of search to use
  startQuickSearch : function(v) {
    var defaultSearchType = null;
    var searchType = null;
        for (var i = 0; i < this.searchTypes.length; i++)
        {
      // this default search will be used if no other searches match
          if (this.searchTypes[i].defaultQuickSearch == true)
          {
        defaultSearchType = this.searchTypes[i];
          }
          else if (this.searchTypes[i].quickSearchCheckHandler.call(this, this, v))
          {
        searchType = this.searchTypes[i];
        break;
      }
    }

    // apply the default search
        if (searchType == null && defaultSearchType != null)
        {
      searchType = defaultSearchType;
    }

        if (searchType != null)
        {
      this.switchSearchType( {
        value : searchType.value
      }, null);

      searchType.quickSearchHandler.call(this, this, v);
      this.startSearch(this, true);
    }
  },
  // apply the bookmark params to page
  applyBookmark : function(bookmark) {
        if (bookmark)
        {
      var parts = decodeURIComponent(bookmark).split('~');

      // if type not specified, simply do a quick search and guess
          if (parts.length == 1)
          {
        this.startQuickSearch(bookmark);
          }
          else if (parts.length > 1)
          {
        this.switchSearchType( {
          value : parts[0]
        }, null);

        var searchType = this.getSearchType(parts[0]);

        searchType.applyBookmarkHandler.call(this, this, parts);
      }
    }
  },
  // get the params to build bookmark
  getBookmark : function() {
    var searchType = this.getSearchType(this.searchTypeButton.value);
    
    var bookmark = searchType.getBookmarkHandler.call(this, this);
    
        if (this.extraData)
        {
      var extras = this.extraData.split( ',' );
      
          if (extras.length > 0)
          {
        bookmark += '~';
            for (var i = 0; i < extras.length; i++)
            {
              if (i > 0)
              {
            bookmark += ',';
          }
          bookmark += extras[i];
        }
      }
    }
    
    return bookmark;
  }
});

// Add the quick search
Sonatype.Events.addListener('searchTypeInit', function(searchTypes, panel) {
  //keyword is the default, we always want first in list
  searchTypes.splice( 0, 0, {
    value : 'quick',
    text : 'Keyword Search',
    scope : panel,
    handler : panel.switchSearchType,
    defaultQuickSearch : true,
    // use the default store
    store : null,
    showArtifactContainer : function( record ) {
              if ('COLLAPSED' == record.get('version') || 'COLLAPSED' == record.get('packaging') || 'COLLAPSED' == record.get('classifier'))
              {
      return false;
              }
              return true;
    },
    //use new column model to hide the source column
    columnModel : new Ext.grid.ColumnModel({
                  columns : [{
          id: 'group',
          header: "Group",
          dataIndex: 'groupId',
          sortable:true
                      }, {
          id: 'artifact',
          header: "Artifact",
          dataIndex: 'artifactId',
          sortable:true
                      }, {
          id: 'version',
          header: "Version",
          dataIndex: 'version',
          sortable:true,
          renderer: panel.grid.formatVersionLink
                      }, {
          id: 'packaging',
          header: "Packaging",
          dataIndex: 'packaging',
          sortable:true,
          renderer: panel.grid.formatPackagingLink
                      }, {
          id: 'classifier',
          header: "Classifier",
          dataIndex: 'classifier',
          sortable:true,
          renderer: panel.grid.formatClassifierLink
                      }]
    }),
    quickSearchCheckHandler : function(panel, value) {
      return true;
    },
    quickSearchHandler : function(panel, value) {
      panel.getTopToolbar().items.itemAt(1).setRawValue(value);
    },
    searchHandler : function(panel) {
      var value = panel.getTopToolbar().items.itemAt(1).getRawValue();

              if (value)
              {
        panel.grid.store.baseParams = {};
        panel.grid.store.baseParams['q'] = value;
        panel.grid.store.baseParams['collapseresults'] = true;
        panel.fetchRecords(panel);
      }
    },
    applyBookmarkHandler : function(panel, data) {
      panel.extraData = null;
      panel.getTopToolbar().items.itemAt(1).setRawValue(data[1]);
      panel.startSearch(panel, false);
    },
    getBookmarkHandler : function(panel) {
      var result = panel.searchTypeButton.value;
      result += '~';
      result += panel.getTopToolbar().items.itemAt(1).getRawValue();
      return result;
    },
    panelItems : [ {
      xtype : 'nexussearchfield',
      name : 'single-search-field',
      searchPanel : panel,
      width : 300
    } ]
  });
});

// Add the classname search
Sonatype.Events.addListener('searchTypeInit', function(searchTypes, panel) {
  searchTypes.push( {
    value : 'classname',
    text : 'Classname Search',
    scope : panel,
    // use the default store
    store : null,
    handler : panel.switchSearchType,
    quickSearchCheckHandler : function(panel, value) {
      return value.search(/^[a-z.]*[A-Z]/) == 0;
    },
    quickSearchHandler : function(panel, value) {
      panel.getTopToolbar().items.itemAt(1).setRawValue(value);
    },
    searchHandler : function(panel) {
      var value = panel.getTopToolbar().items.itemAt(1).getRawValue();

              if (value)
              {
        panel.grid.store.baseParams = {};
        panel.grid.store.baseParams['cn'] = value;
        panel.fetchRecords(panel);
      }
    },
    applyBookmarkHandler : function(panel, data) {
      panel.getTopToolbar().items.itemAt(1).setRawValue(data[1]);
      panel.startSearch(panel, false);
    },
    getBookmarkHandler : function(panel) {
      var result = panel.searchTypeButton.value;
      result += '~';
      result += panel.getTopToolbar().items.itemAt(1).getRawValue();

      return result;
    },
    panelItems : [ {
      xtype : 'nexussearchfield',
      name : 'single-search-field',
      searchPanel : panel,
      width : 300
    } ]
  });
});

// Add the gav search
Sonatype.Events.addListener('searchTypeInit', function(searchTypes, panel) {
  var enterHandler = function(f, e) {
        if (e.getKey() == e.ENTER)
        {
      this.startSearch(this, true);
    }
  };

  var gavPopulator = function(panel, data) {
    panel.extraData = null;
    
    // groupId
        if (data.length > 1)
        {
      panel.getTopToolbar().items.itemAt(2).setRawValue(data[1]);
    }
    // artifactId
        if (data.length > 2)
        {
      panel.getTopToolbar().items.itemAt(5).setRawValue(data[2]);
    }
    // version
        if (data.length > 3)
        {
      panel.getTopToolbar().items.itemAt(8).setRawValue(data[3]);
    }
    // packaging
        if (data.length > 4)
        {
      panel.getTopToolbar().items.itemAt(11).setRawValue(data[4]);
    }
    // classifier
        if (data.length > 5)
        {
      panel.getTopToolbar().items.itemAt(14).setRawValue(data[5]);
    }
    // extra params, comma seperated list of params
        if (data.length > 6)
        {
      panel.extraData = data[6];
    }
  }

  searchTypes.push( {
    value : 'gav',
    text : 'GAV Search',
    scope : panel,
    // use the default store
    store : null,
    showArtifactContainer : function( record ) {
              if ('COLLAPSED' == record.get('version') || 'COLLAPSED' == record.get('packaging') || 'COLLAPSED' == record.get('classifier'))
              {
        return false;
      }
      return true;
    },
    handler : panel.switchSearchType,
    quickSearchCheckHandler : function(panel, value) {
      return value.indexOf(':') > -1;
    },
    quickSearchHandler : function(panel, value) {
      var parts = value.split(':');
      var data = [ 'gav' ];
              for (var i = 0; i < parts.length; i++)
              {
        data.push(parts[i]);
      }
      gavPopulator(panel, data);
    },
    searchHandler : function(panel) {
      var reverseSortResults = false; 
      this.grid.store.baseParams = {};

      // groupId
    var v = panel.getTopToolbar().items.itemAt(2).getRawValue();
              if (v)
              {
      panel.grid.store.baseParams['g'] = v;
    }
    // artifactId
    v = panel.getTopToolbar().items.itemAt(5).getRawValue();
              if (v)
              {
      panel.grid.store.baseParams['a'] = v;
    }
    // version
    v = panel.getTopToolbar().items.itemAt(8).getRawValue();
              if (v)
              {
      panel.grid.store.baseParams['v'] = v;
    }
    // packaging
    v = panel.getTopToolbar().items.itemAt(11).getRawValue();
              if (v)
              {
      panel.grid.store.baseParams['p'] = v;
    }
    // classifier
    v = panel.getTopToolbar().items.itemAt(14).getRawValue();
              if (v)
              {
      panel.grid.store.baseParams['c'] = v;
    }
    
    panel.grid.store.baseParams['collapseresults'] = true;
    
    // go through the extras and process them.
              if (panel.extraData)
              {
      var extras = panel.extraData.split( ',' );
      
                for (var i = 0; i < extras.length; i++)
                {
        // from keyword search
                  if (extras[i] == 'kw')
                  {
          reverseSortResults = true;
        }
                  else if (extras[i] == 'versionexpand')
                  {
          panel.grid.store.baseParams['versionexpand'] = true;
        }
                  else if (extras[i] == 'packagingexpand')
                  {
          panel.grid.store.baseParams['packagingexpand'] = true;
        }
                  else if (extras[i] == 'classifierexpand')
                  {
          panel.grid.store.baseParams['classifierexpand'] = true;
        }
      }
    }

              if (panel.grid.store.baseParams['g'] == null && panel.grid.store.baseParams['a'] == null && panel.grid.store.baseParams['v'] == null)
              {
      panel.setWarningLabel('A group, an artifact or a version is required to run a search.');
      return;
    }

    panel.clearWarningLabel();

    panel.fetchRecords(panel, reverseSortResults);
  },
  applyBookmarkHandler : function(panel, data) {
    gavPopulator(panel, data);
    panel.startSearch(this, false);
  },
  getBookmarkHandler : function(panel) {
    var result = panel.searchTypeButton.value;
    // groupId
    result += '~';
    var v = panel.getTopToolbar().items.itemAt(2).getRawValue();
              if (v)
              {
      result += v;
    }
    // artifactId
    result += '~';
    v = panel.getTopToolbar().items.itemAt(5).getRawValue();
              if (v)
              {
      result += v;
    }
    // version
    result += '~';
    v = panel.getTopToolbar().items.itemAt(8).getRawValue();
              if (v)
              {
      result += v;
    }
    // packaging
    result += '~';
    v = panel.getTopToolbar().items.itemAt(11).getRawValue();
              if (v)
              {
      result += v;
    }
    // classifier
    result += '~';
    v = panel.getTopToolbar().items.itemAt(14).getRawValue();
              if (v)
              {
      result += v;
    }
    
    return result;
  },
  panelItems : [ 'Group:', {
    xtype : 'textfield',
    id : 'gavsearch-group',
    size : 80,
    listeners : {
      'specialkey' : {
        fn : enterHandler,
        scope : panel
      }
    }
  }, {
    xtype : 'tbspacer'
  }, 'Artifact:', {
    xtype : 'textfield',
    id : 'gavsearch-artifact',
    size : 80,
    listeners : {
      'specialkey' : {
        fn : enterHandler,
        scope : panel
      }
    }
  }, {
    xtype : 'tbspacer'
  }, 'Version:', {
    xtype : 'textfield',
    id : 'gavsearch-version',
    size : 80,
    listeners : {
      'specialkey' : {
        fn : enterHandler,
        scope : panel
      }
    }
  }, {
    xtype : 'tbspacer'
  }, 'Packaging:', {
    xtype : 'textfield',
    id : 'gavsearch-packaging',
    size : 80,
    listeners : {
      'specialkey' : {
        fn : enterHandler,
        scope : panel
      }
    }
  }, {
    xtype : 'tbspacer'
  }, 'Classifier:', {
    xtype : 'textfield',
    id : 'gavsearch-classifier',
    size : 80,
    listeners : {
      'specialkey' : {
        fn : enterHandler,
        scope : panel
      }
    }
  }, {
    xtype : 'tbspacer'
  }, {
    icon : Sonatype.config.resourcePath + '/images/icons/search.gif',
    cls : 'x-btn-icon',
    scope : panel,
    handler : function() {
      this.startSearch(this, true);
    }
  } ]
  });
});

// Add the checksum search
Sonatype.Events.addListener('searchTypeInit', function(searchTypes, panel) {
      if (Sonatype.lib.Permissions.checkPermission('nexus:identify', Sonatype.lib.Permissions.READ))
      {
    searchTypes.push( {
      value : 'checksum',
      text : 'Checksum Search',
      scope : panel,
      // use the default store
      store : null,
      handler : panel.switchSearchType,
      quickSearchCheckHandler : function(panel, value) {
        return value.search(/^[0-9a-f]{40}$/) == 0;
      },
      quickSearchHandler : function(panel, value) {
        panel.getTopToolbar().items.itemAt(1).setRawValue(value);
      },
      searchHandler : function(panel) {
        var value = panel.getTopToolbar().items.itemAt(1).getRawValue();

                if (value)
                {
          panel.grid.store.baseParams = {};
          panel.grid.store.baseParams['sha1'] = value;
          panel.fetchRecords(panel);
        }
      },
      applyBookmarkHandler : function(panel, data) {
        panel.getTopToolbar().items.itemAt(1).setRawValue(data[1]);
        panel.startSearch(panel, false);
      },
      getBookmarkHandler : function(panel) {
        var result = panel.searchTypeButton.value;
        result += '~';
        result += panel.getTopToolbar().items.itemAt(1).getRawValue();

        return result;
      },
              panelItems : [{
            xtype : 'nexussearchfield',
            name : 'single-search-field',
            searchPanel : panel,
            width : 300
                  }, {
            xtype : Ext.isGecko3 ? 'button' : 'browsebutton',
            text : 'Browse...',
            searchPanel : panel,
            tooltip : 'Click to select a file. It will not be uploaded to the ' + 'remote server, an SHA1 checksum is calculated locally and sent to ' + 'Nexus to find a match. This feature requires Java applet ' + 'support in your web browser.',
            handler : function(b) {
                      if (!document.digestApplet)
                      {
                        b.searchPanel.grid.fetchMoreBar.addText('<div id="checksumContainer" style="width:10px">' + '<applet code="org/sonatype/nexus/applet/DigestApplet.class" ' + 'archive="' + Sonatype.config.resourcePath + '/digestapplet.jar" '
                            + 'width="1" height="1" name="digestApplet"></applet>' + '</div>');
              }

              var filename = null;

                      if (Ext.isGecko3)
                      {
                filename = document.digestApplet.selectFile();
                      }
                      else
                      {
                var fileInput = b.detachInputFile();
                filename = fileInput.getValue();
              }

                      if (!filename)
                      {
                return;
              }

              b.disable();

              var setFilenameLabel = function(panel, s) {
                        if (panel.filenameLabel)
                        {
                  panel.filenameLabel.destroy();
                }
                panel.filenameLabel = s ? panel.searchToolbar.addText('<span style="color:#808080;">' + s + '</span>') : null;
              };

              setFilenameLabel(b.searchPanel, 'Calculating checksum...');

              var f = function(b, filename) {
                var sha1 = 'error calculating checksum';
                        if (document.digestApplet)
                        {
                  sha1 = document.digestApplet.digest(filename);
                }

                b.searchPanel.getTopToolbar().items.itemAt(1).setRawValue(sha1);
                setFilenameLabel(b.searchPanel, filename);
                b.enable();
                b.searchPanel.startSearch(b.searchPanel, true);
              }
              f.defer(200, b, [ b, filename ]);
            }
          } ]
    });
  }
});

// This container will host both the repository browser and the artifact information panel
Sonatype.repoServer.RepositoryIndexBrowserContainer = function( config ) {
  var config = config || {};
  var defaultConfig = {};
  Ext.apply( this, config, defaultConfig );
  
  var items = [];
  
  this.repositoryBrowser = new Sonatype.repoServer.IndexBrowserPanel({
    payload: this.payload,
    tabTitle: this.tabTitle,
    region: 'center',
    nodeClickEvent: 'indexNodeClickedEvent',
    nodeClickPassthru: {
      container: this
    }
  });
  
  this.artifactContainer = new Sonatype.repoServer.ArtifactContainer({
    collapsible: true,
    collapsed: true,
    region: 'east',
    split: true,
    width: 500,
    halfSize: true
  });
  
  items.push(this.repositoryBrowser);
  items.push(this.artifactContainer);
  
  Sonatype.repoServer.RepositoryIndexBrowserContainer.superclass.constructor.call( this, {
    layout: 'border',
    //this hideMode causes the tab to properly render when coming back from hidden
    hideMode: 'offsets',
    items: items
  });
};

Ext.extend( Sonatype.repoServer.RepositoryIndexBrowserContainer, Ext.Panel, {
  
});

// Add the browse storage and browse index panels to the repo
Sonatype.Events.addListener( 'repositoryViewInit', function( cardPanel, rec ) {
  if ( rec.data.resourceURI 
      && rec.data.repoType != 'virtual' 
      && rec.data.format == 'maven2' ) {
    var panel = new Sonatype.repoServer.RepositoryIndexBrowserContainer( { 
      payload: rec,
      name: 'browseindex',
      tabTitle: 'Browse Index'
    } );
    
    if ( cardPanel.items.getCount() > 0 ) {
      cardPanel.insert( 1, panel );
    }
    else {
      cardPanel.add( panel );
    }
  }
} );

Sonatype.Events.addListener( 'indexNodeClickedEvent', function( node, passthru ) {
  if ( passthru 
      && passthru.container ) {
    if ( node 
        && node.isLeaf() ) {
      passthru.container.artifactContainer.updateArtifact({
        groupId: node.attributes.groupId,
        artifactId: node.attributes.artifactId,
        version: node.attributes.version,
        repoId: node.attributes.repositoryId,
        classifier: node.attributes.classifier,
        extension: node.attributes.extension,
        artifactLink: node.attributes.artifactUri,
        pomLink: node.attributes.pomUri,
        nodeName: node.attributes.nodeName
      });
    }
    else {
      passthru.container.artifactContainer.collapse();
    }
  }
});
/*
 * Sonatype Nexus (TM) Open Source Version.
 * Copyright (c) 2008 Sonatype, Inc. All rights reserved.
 * Includes the third-party code listed at http://nexus.sonatype.org/dev/attributions.html
 * This program is licensed to you under Version 3 only of the GNU General Public License as published by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License Version 3 for more details.
 * You should have received a copy of the GNU General Public License Version 3 along with this program.
 * If not, see http://www.gnu.org/licenses/.
 * Sonatype Nexus (TM) Professional Version is available from Sonatype, Inc.
 * "Sonatype" and "Sonatype Nexus" are trademarks of Sonatype, Inc.
 */
// must pass in feedUrl that's local to our domain, 'cause we ain't got no proxy yet

// config: feedUrl required

Sonatype.SearchStore = function(config) {
  var config = config || {};
  var defaultConfig = {
    searchUrl: Sonatype.config.servicePath + '/data_index'
  };
  Ext.apply(this, config, defaultConfig);
  
  Sonatype.SearchStore.superclass.constructor.call(this, {
    proxy: new Ext.data.HttpProxy({
      url: this.searchUrl,
      method: 'GET'
    }),
    reader: new Ext.data.JsonReader(
      {
        root: 'data',
        totalProperty: 'totalCount'
      },
      Ext.data.Record.create([
        {name:'groupId'},
        {name:'artifactId'},
        {name:'version'},
        {name:'repoId'},
        {name:'resourceURI'},
        {name:'contextId'},
        {name:'classifier'},
        {name:'packaging'},
        {name:'extension'},
        {name:'pomLink'},
        {name:'artifactLink'},
        {name:'highlightedFragment'}
      ])
    ),
    listeners: {
      'beforeload': {
        fn: function( store, options ) {
          store.proxy.getConnection().on( 'requestcomplete',
            function( conn, response, options ) {
              if ( response.responseText ) {
                var statusResp = Ext.decode(response.responseText);
                if ( statusResp ) {
                  this.grid.totalRecords = statusResp.totalCount;
                  if ( statusResp.tooManyResults ) {
                    this.grid.setWarningLabel( 'Too many results, please refine the search condition.' );
                  }
                  else {
                    this.grid.clearWarningLabel();
                  }
                }
              }
            }, this, { single: true } );
          return true;
        },
        scope: this
      },
      'load': {
        fn: function( store, records, options ) {
          this.grid.updateRowTotals( this.grid );
        },
        scope: this
      }
    }
  });
};

Ext.extend(Sonatype.SearchStore, Ext.data.Store, {
});

Sonatype.repoServer.SearchResultGrid = function(config) {

  Ext.apply(this, config);
  
  this.sp = Sonatype.lib.Permissions;
  
  this.totalRecords = 0;
  
  this.defaultStore = new Sonatype.SearchStore({
    grid: this
  });
  
  this.store = this.defaultStore;
  
  this.defaultColumnModel = new Ext.grid.ColumnModel({
    columns: [
      {
        id: 'source',
        header: "Source Index",
        dataIndex: 'contextId',
        sortable:true
      },
      {
        id: 'group',
        header: "Group",
        dataIndex: 'groupId',
        sortable:true
      },
      {
        id: 'artifact',
        header: "Artifact",
        dataIndex: 'artifactId',
        sortable:true
      },
      {
        id: 'version',
        header: "Version",
        dataIndex: 'version',
        sortable:true,
        renderer: this.formatVersionLink
      },
      {
        id: 'packaging',
        header: "Packaging",
        dataIndex: 'packaging',
        sortable:true,
        renderer: this.formatPackagingLink
      },
      {
        id: 'classifier',
        header: "Classifier",
        dataIndex: 'classifier',
        sortable:true,
        renderer: this.formatClassifierLink
      }
    ]
  });
  
  this.colModel = this.defaultColumnModel;

  this.clearButton = new Ext.Button({
    text: 'Clear Results',
    icon: Sonatype.config.resourcePath + '/images/icons/clear.gif',
    cls: 'x-btn-text-icon',
    handler: this.clearResults,
    disabled: true,
    scope: this
  });

  this.fetchMoreBar = new Ext.Toolbar({
    ctCls: 'search-all-tbar',
    items: [ 
      'Displaying 0 records',
      { xtype: 'tbspacer' },
      this.clearButton
    ]
  });

  Sonatype.repoServer.SearchResultGrid.superclass.constructor.call(this, {
      region: 'center',
      id: 'search-result-grid',
      loadMask: {msg:'Loading Results...'},
      stripeRows: true,
      sm: new Ext.grid.RowSelectionModel({
          singleSelect: true
      }),
      
      bbar: this.fetchMoreBar,

      viewConfig: {
          forceFit:true,
          enableRowBody:true,
          getRowClass : this.applyRowClass
      },
      
      listeners : {
          render : {
            fn : function(grid) {
              grid.body.on({
                'mousedown' : function(e, t) {
                  var i = t.getAttribute('index');
                  this.toggleExtraInfo(parseInt(i, 10));
                  e.stopEvent();
                  return false;
                },
                'click' : function(e, t) {
                  e.stopEvent();
                  return false;
                },
                delegate : 'a.pom-link',
                scope : grid
              });
                  
              var store = grid.getStore ();
			  var view = grid.getView ();
			  grid.tip = new Ext.ToolTip ({
			    target: view.mainBody,
			    delegate: '.x-grid3-row',
                maxWidth: 500,
			    trackMouse: true,
			    renderTo: document.body,
			    listeners: {
			      beforeshow: function (tip) {
			        var rowIndex = view.findRowIndex (tip.triggerElement);
                    var record = store.getAt( rowIndex );
                    
                    if ( !record ) {
                      return false;
                    }
                    
                    var highlightedFragment = record.get('highlightedFragment');
                    
                    if ( Ext.isEmpty( highlightedFragment ) ) {
                      return false;
                    }
                    
			        tip.body.dom.innerHTML = highlightedFragment;
			      }
			    }
			  });
            },
            scope : this
          }
        }
  });
};

Ext.extend(Sonatype.repoServer.SearchResultGrid, Ext.grid.GridPanel, {
  formatCellLink : function( record, store, fieldName ) {
    if ( 'COLLAPSED' != record.get( fieldName ) ) {
      return record.get( fieldName );
    }
    
    var ver = record.get( 'version' );
    var gid = record.get( 'groupId' );
    var aid = record.get( 'artifactId' );
    var pac = record.get( 'packaging' );
    var clas = record.get( 'classifier' );
    
    var extras = '~kw';
    var aggregatedCount = 0;
    if ( 'COLLAPSED' == ver ) {
      aggregatedCount++;
      if ( 'version' == fieldName ) {
        extras += ',versionexpand';
      }
      ver = '';
    }
    else {
      extras += ",versionexpand";
    }
    if ( 'COLLAPSED' == pac ) {
      aggregatedCount++;
      if ( 'packaging' == fieldName ) {
        extras += ',packagingexpand';
      }
      pac = '';
    }
    else {
      extras += ",packagingexpand";
      if ( Ext.isEmpty( pac ) ) {
        pac = 'N/P';
      }
    }
    if ( 'COLLAPSED' == clas ) {
      aggregatedCount++;
      if ( 'classifier' == fieldName ) {
        extras += ',classifierexpand';
      }
      clas = '';
    }
    else {
      extras += ",classifierexpand";
      //special case for classifier, if the classifier is real value (NOT COLLAPSED) need to set that
      //if empty, need to set N/P
      if ( Ext.isEmpty( clas ) ) {
        clas = 'N/P';
      }
    }
    
    // if 0 then is gav search being performed, so simply return string
    if ( aggregatedCount == 0 ) {
      return value;
    }
    // if anything is aggregated, add the drill down link
    else {
      return '<a href="#nexus-search;gav~'+gid+'~'+aid+'~'+ver+'~'+pac+'~'+clas+extras+' " onmousedown="cancel_bubble(event)" onclick="cancel_bubble(event); return true;">Drill Down</a>';
    }
  },
  formatVersionLink : function(value, p, record, rowIndex, colIndex, store) {
    return store.grid.formatCellLink( record, store, 'version' );
  },
  formatPackagingLink : function( value, p, record, rowIndex, colIndex, store) {
    return store.grid.formatCellLink( record, store, 'packaging' );
  },  
  formatClassifierLink : function( value, p, record, rowIndex, colIndex, store) {
    return store.grid.formatCellLink( record, store, 'classifier' );
  },
  
  switchStore : function( grid, store, columnModel ) {
    if ( store == null ) {
      store = grid.defaultStore;
    }
    
    if ( columnModel == null ) {
      columnModel = grid.defaultColumnModel;
    }
    
    if ( store ) {
      this.clearResults();
    }
    
    grid.reconfigure( store, columnModel );
  },
  toggleExtraInfo : function(rowIndex){
    var rowEl = new Ext.Element(this.getView().getRow(rowIndex));
    var input = rowEl.child('.copy-pom-dep', true);
    input.select(); //@todo: why won't this field highlight?!!!!
    rowEl.toggleClass('x-grid3-row-expanded');
  },

  // within this function "this" is actually the GridView
  applyRowClass: function(record, rowIndex, p, ds) {
    var xf = Ext.util.Format;
    var xmlDep = Sonatype.repoServer.RepoServer.pomDepTmpl.apply(record.data);
    //note: wrapper div with overflow:auto and autocomplete="off" are attempts to avoid a FF bug
    p.body = '<span>POM Dependency: </span><input class="copy-pom-dep" type="text" autocomplete="off" value="'+xmlDep+'"/>';
    return 'x-grid3-row-collapsed';
  },
  
  updateRowTotals: function( p ) {
    var count = p.store.getCount();

    p.clearButton.setDisabled( count == 0 );
    
    if ( count == 0 || count > p.totalRecords ) {
      p.totalRecords = count;
    }
    
    p.fetchMoreBar.items.items[0].destroy();
    p.fetchMoreBar.items.removeAt( 0 );
    p.fetchMoreBar.insertButton( 0, new Ext.Toolbar.TextItem( 'Displaying ' + count + ' records' ) );
  },

  setWarningLabel: function( s ) {
    this.searchPanel.setWarningLabel( s );
  },

  clearWarningLabel: function() {
    this.searchPanel.clearWarningLabel();
  },
  
  clearResults: function() {
    this.store.baseParams = {};
    this.store.removeAll();
    delete this.store.baseParams['dir'];
    delete this.store.baseParams['sort'];
    delete this.store.sortInfo;
    delete this.view.sortState;
    this.store.sortToggle = {};
    this.view.mainHd.select('td').removeClass(this.sortClasses);
    this.fireEvent('sortchange', this.grid, null);
    this.updateRowTotals( this );
    this.clearWarningLabel();
  }
});

cancel_bubble = function (e) {
	if (!e) var e = window.event;
	e.cancelBubble = true;
	if (e.stopPropagation) e.stopPropagation(); 
}

var SEARCH_FIELD_CONFIG = {
  xtype: 'trigger',
  triggerClass: 'x-form-search-trigger',
  listeners: {
    'specialkey': {
      fn: function(f, e){
        if(e.getKey() == e.ENTER){
          this.onTriggerClick();
        }
      }
    }
  },
  onTriggerClick: function(a,b,c){
    var v = this.getRawValue();
    if ( v.length > 0 ) {
      var panel = Sonatype.view.mainTabPanel.addOrShowTab(
          'nexus-search', Sonatype.repoServer.SearchPanel, { title: 'Search' } );
      panel.startQuickSearch( v );
    }
  }
};

Sonatype.Events.addListener( 'nexusNavigationInit', function( nexusPanel ) {
  if ( Sonatype.lib.Permissions.checkPermission( 'nexus:index', Sonatype.lib.Permissions.READ ) ) {
    nexusPanel.add({
      title: 'Artifact Search',
      id: 'st-nexus-search',
      items: [
        Ext.apply( {
          repoPanel: this,
          id: 'quick-search--field',
          width: 140
        }, SEARCH_FIELD_CONFIG ),
        {
          title: 'Advanced Search',
          tabCode: Sonatype.repoServer.SearchPanel,
          tabId: 'nexus-search',
          tabTitle: 'Search'
        }
      ]
    });
  }
});

Sonatype.Events.addListener( 'welcomePanelInit', function( repoServer, welcomePanelConfig ) {
  if ( Sonatype.lib.Permissions.checkPermission( 'nexus:index', Sonatype.lib.Permissions.READ ) ) {
    welcomePanelConfig.items.push( {
      layout: 'form',
      border: false,
      frame: false,
      labelWidth: 10,
      items: [
        {
          border: false,
          html: '<div class="little-padding">' +
            'Type in the name of a project, class, or artifact into the text box ' +
            'below, and click Search. Use "Advanced Search" on the left for more options.' +
            '</div>'
        },
        Ext.apply( {
          repoPanel: repoServer,
          id: 'quick-search-welcome-field',
          anchor: '-10',
          labelSeparator: ''
        }, SEARCH_FIELD_CONFIG )
      ]
    } );
  }
});

Sonatype.Events.addListener( 'welcomeTabRender', function() {
  var c = Ext.getCmp( 'quick-search-welcome-field' );
  if ( c ) {
    c.focus( true, 100 );
  }
});
Sonatype.repoServer.IndexBrowserPanel = function( config ) {
  var config = config || {};
  var defaultConfig = { };
  Ext.apply( this, config, defaultConfig );

  Sonatype.repoServer.IndexBrowserPanel.superclass.constructor.call( this, {
    nodeIconClass: 'x-tree-node-nexus-icon',
    useNodeIconClassParam: 'locallyAvailable',
    autoExpandRoot: false,
    url: this.payload.data.resourceURI + '/index_content'
  } );
};

Ext.extend( Sonatype.repoServer.IndexBrowserPanel, Sonatype.panels.TreePanel, {
} );
